{% extends "base.html" %}
{% block content %}
<div class="alert alert-warning top1em">
  <span class="pficon-layered">
    <span class="pficon pficon-warning-triangle"></span>
    <span class="pficon pficon-warning-exclamation"></span>
  </span>
  {{ message }}
</div>
{% endblock %}