{% extends "core.html" %}
{% block html_class %}login-pf{% endblock %}
{% block body %}
<span id="badge">
  <img alt="Red Hat® logo" src="../../static/img/logo.svg">
</span>
<div class="container">
    <div class="row">
      <div class="col-sm-12">
          <h3>Sign in to Preupgrade assistant UI</h3>
      </div><!--/.col-*-->
      <div class="col-sm-7 col-md-6 col-lg-5">
        {% if form.errors %}
        <div class="alert alert-danger">
          <span class="pficon-layered">
            <span class="pficon pficon-error-octagon"></span>
            <span class="pficon pficon-error-exclamation"></span>
          </span>
          Your username and password didn't match. Please try again.
        </div>
        {% endif %}
        <form class="form-horizontal" role="form" method="post" action="{% url 'auth-login' %}">
          {% csrf_token %}
          <div class="form-group">
            <label for="inputUsername" class="col-sm-2 col-md-2 control-label">Username</label>
            <div class="col-sm-10 col-md-10">
              <input type="text" class="form-control" id="inputUsername" placeholder="" tabindex="1" name="username">
            </div>
          </div>
          <div class="form-group">
            <label for="inputPassword" class="col-sm-2 col-md-2 control-label">Password</label>
            <div class="col-sm-10 col-md-10">
              <input type="password" class="form-control" id="inputPassword" placeholder="" tabindex="2" name="password">
            </div>
          </div>
          <div class="form-group">
            <div class="col-sm-12 col-md-12 submit">
              <button type="submit" class="btn btn-primary btn-lg" tabindex="4">Log In</button>
            </div>
          </div>
          <input type="hidden" name="next" value="{% url "results-list" %}" />
        </form>
      </div><!--/.col-*-->
      {% if right_text %}
      <div class="col-sm-5 col-md-6 col-lg-7 details">
        {{ right_text }}
      </div><!--/.col-*-->
      {% endif %}
    </div><!--/.row-->
</div><!--/.container-->
{% endblock %}
