{% extends "base.html" %}

{% block nav_users %}active{% endblock %}
{% block content %}
<div class="row col-sm-7 col-md-6 col-lg-5">
      <h3>{% block title %}Delete User{% endblock %}</h3>
      {% if form.errors %}
      <div class="alert alert-danger">
        <span class="pficon-layered">
          <span class="pficon pficon-error-octagon"></span>
          <span class="pficon pficon-error-exclamation"></span>
        </span>
        {% for key, value in form.errors.items %}
          {% for error in value %}
            <div>{{ error }}</div>
          {% endfor %}
        {% endfor %}
      </div>
      {% endif %}
      <form class="form-horizontal" role="form" method="post">
        {% csrf_token %}
        <div class="form-group">
          <div class="col-sm-12 col-md-12">
            Are you sure you want to delete user "{{ object }}"?
          </div>
          <div class="col-sm-12 col-md-12 submit">
            <a class="btn btn-default btn-lg" href="{% url 'auth-list' %}">Back</a>
            <button type="submit" class="btn btn-primary btn-lg" tabindex="2">Confirm</button>
          </div>
        </div>
      </form>
</div><!--/.row-->
{% endblock %}
