{% extends "core.html" %}
{% block html_class %}login-pf{% endblock %}
{% block body %}
<span id="badge">
  <img alt="Red Hat® logo" src="../../static/img/logo.svg">
</span>
<div class="container">
    <div class="row">
      <div class="col-sm-12">
          <h3>Logged out</h3>
          <p><a href="{% url 'auth-login' %}">login again</a></p>
      </div><!--/.col-*-->
    </div><!--/.row-->
</div><!--/.container-->
{% endblock %}
