<!DOCTYPE html>
<html lang="en" class="{% block html_class %}{% endblock %}">
<head>
    <meta charset="utf-8">
    <title>{% block title %}{{ title }}{% endblock %}</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="Preupgrade assistant UI.">
    <meta name="author" content="Red Hat">

    <link rel="stylesheet" href="{{ STATIC_URL }}css/bootstrap.min.css" type="text/css">
    <link rel="stylesheet" href="{{ STATIC_URL }}css/rcue.min.css" type="text/css" />
    <link rel="stylesheet" href="{{ STATIC_URL }}css/bootstrap-select.min.css" type="text/css">
    <link rel="stylesheet" href="{{ STATIC_URL }}css/style.css" type="text/css" />

    {% block head %}
    {% endblock %}
</head>
<body>
    {% block body %}
    {% endblock %}

    <script src="{{ STATIC_URL }}js/jquery-2.0.3.min.js"></script>
    <script src="{{ STATIC_URL }}js/patternfly.min.js"></script>
    <script src="{{ STATIC_URL }}js/bootstrap.min.js"></script>
    <script src="{{ STATIC_URL }}js/bootstrap-select.min.js"></script>
    {% block js %}
    {% endblock %}
</body>
</html>
